/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.combine;

import java.util.Collection;

/**
 * <div>
 * A strategy that creates a collection of callback-records in which all
 * possible combinations of two individual callback-parameters will be
 * represented. The strategy also tries to accomplish this task while keeping
 * the number of callback-records as low as possible. This works alright but
 * there is no guarantee that the lowest number of callback-records is always
 * found but it seems to work flawlessly when the ValuesCollection contains
 * three arrays and OK when there are four or more arrays.
 * </div>
 * <div>
 * If there is some combinatory expert available out there than feel free to
 * provide some optimized algorithms for this. The current algorithms are pretty
 * hard to understand and to some extent they also rely on chance.
 * </div>
 * 
 * @author Henrik Kaipe
 */
public class CombineAllPossible2Combinations extends AbstractCombineStrategy {

    protected Collection combineInternal(ValuesCollection vc) {
        return AllPossible2Combinations.combine(vc, getRandomSeed());
    }
}
