/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.combine;

import java.util.Iterator;

/**
 * Used by {@link CombineCompletely}
 *
 * @author Henrik Kaipe
 */
class CompleteCombinator {

    static Iterator combine(final ValuesCollection vc) {
        
        return new Iterator() {
            
            boolean hasNext = true;
            final int[] indexes = new int[vc.size()];

            public boolean hasNext() {
                return hasNext;
            }

            public Object next() {
                final Object[] record = new Object[indexes.length];
                for (int i = 0 ; i < record.length ; ++i) {
                    record[i] = vc.get(i)[indexes[i]];
                }
                hasNext = inc(vc, indexes);
                return record;
            }

            public void remove(){/*Already removed - has never been added :) */}
        };
    }

    private static boolean inc(ValuesCollection vc, int[] indexes) {
        for (int i = indexes.length - 1 ; 0 <= i ; --i) {
            if (++indexes[i] < vc.get(i).length) {
                return true;
            } else {
                indexes[i] = 0;
            }
        }
        return false;
    }
}
