/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.combine.annotation;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import org.callbackparams.combine.CombineStrategy;

/**
 * Using this annotation is the recommended practise for configuring the
 * automated combining of callback-values into callback-records.
 * @see CombineStrategy
 * 
 * @author Henrik Kaipe
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.TYPE)
@Inherited
@Documented
public @interface CombineConfig {

    /**
     * Specifies the combine-strategy implementation class.
     * The specified class must have a default (no-arg) constructor.
     */
    Class<? extends CombineStrategy> strategy();

    /**
     * @see CombineStrategy#setMaxCount(int)
     */
    int maxCount() default -1;

    /**
     * @see CombineStrategy#setRandomSeed(long)
     */
    long randomSeed() default -1;
}
