/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.junit4;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import org.callbackparams.junit4.support.BddTestTemplate;
import org.callbackparams.bdd.support.SuperClassChanger;
import org.junit.Test;

/**
 * @author Henrik Kaipe
 */
public class BddRunner extends CallbackParamsRunner {

    public BddRunner(Class<?> testClass) throws Exception {
        super(asBddTest(testClass));
    }

    private static Class<?> asBddTest(Class<?> testClass) throws Exception {
        validate(testClass);
        return new SuperClassChanger(BddTestTemplate.class)
                .asSubclassOf(testClass);
    }

    private static void validate(Class<?> testClass) throws Exception {
        List<String> testAnnotatedMethods = new ArrayList<String>();
        for (Method m : testClass.getMethods()) {
            if (m.isAnnotationPresent(Test.class)) {
                testAnnotatedMethods.add(m.toGenericString());
            }
        }
        if (false == testAnnotatedMethods.isEmpty()) {
            throw new Exception(
                    "BddRunner-test must not have any @Test-methods: "
                    + testAnnotatedMethods);
        }
    }
}
