/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.junit4.support;

import org.callbackparams.internal.template.TestrunCallbacks;
import org.callbackparams.bdd.support.GenericBddRunner;
import org.junit.Test;

/**
 * Works as byte-code template that extends the actual test-class with
 * a test-method that makes sure the entire test will be executed as
 * a BDD test.
 *
 * @author Henrik Kaipe
 */
public class BddTestTemplate
extends NullPointerException { /*Just a dummy super-class*/

    public BddTestTemplate() throws Exception {}

    /**
     * A test-method implementation that will make sure the entire test will
     * be executed as BDD test. Please note that the method will work
     * as test-method when the actual test uses the JUnit-3.x API.
     *
     * @param dummyCallback makes method signature unique
     *        - so that no method is accidently overridden
     * @throws java.lang.Throwable the raw test-failure or test-error
     */
    public void test(DummyInterface dummyCallback) throws Throwable {
        GenericBddRunner.run((TestrunCallbacks)(Object)this);
    }

    /**
     * Works as test-method when the actual test uses JUnit-4.x API.
     * It simply calls method {@link #test(DummyInterface)}, which
     * handles the actual test-execution.
     *
     * @see #test(DummyInterface)
     */
    @Test public void _(DummyInterface dummyCallback) throws Throwable {
        test(null);
    }
}
