/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.junit4.support;

import java.util.List;
import org.callbackparams.internal.template.CallbackRecord;
import org.junit.runners.model.FrameworkMethod;

/**
 * This subclassing of JUnit's FrameworkMethod is to be initiated with callback-
 * arguments. The callback-arguments will be passed to the underlying method
 * when invoked by a test-runner.
 * 
 * @author Henrik Kaipe
 */
public class CallbackFrameworkMethod extends FrameworkMethod {

    private final CallbackMethod callbackMethod;
    private CallbackRecord testRunCallbackRecord;

    public CallbackFrameworkMethod(CallbackMethod callbackMethod) {
        super(callbackMethod.asReflectMethod());
        this.callbackMethod = callbackMethod;
    }

    public void setArgumentRecord(CallbackRecord testRunCallbackRecord) {
        this.testRunCallbackRecord = testRunCallbackRecord;
    }

    @Override
    public String getName() {
        return super.getName() + this.testRunCallbackRecord;
    }

    @Override
    public Object invokeExplosively(Object target, Object... params)
    throws Throwable {
        return super.invokeExplosively(target,
                callbackMethod.createMethodArguments(testRunCallbackRecord));
    }

    @Override
    public void validatePublicVoidNoArg(
            boolean isStatic, List<Throwable> errors) {
        validatePublicVoid(isStatic, errors);
    }
}
