/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.junit4.support;

import org.callbackparams.internal.template.CallbackRecord;
import org.callbackparams.internal.template.CallbackRef;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import org.callbackparams.CallbackControlPanel;

/**
 * Represents a method with callback parameters.
 * 
 * @author Henrik Kaipe
 */
public class CallbackMethod {

    private final Method method;
    private final CallbackRef[] params;

    CallbackMethod(Method method) {
        this.method = method;
        final Class[] rawParamTypes = method.getParameterTypes();
        this.params = new CallbackRef[rawParamTypes.length];
        for (int i = 0 ; i < this.params.length ; ++i) {
            if (CallbackControlPanel.class != rawParamTypes[i]) {
                this.params[i] = CallbackRef.newInstance(method, i);
            }
        }

        if (false == method.isAccessible()) {
            try {
                method.setAccessible(true);
            } catch (SecurityException ignore) {
                /* Ignored */
            }
        }
    }

    Method asReflectMethod() {
        return method;
    }

    Object[] createMethodArguments(final CallbackRecord callbackRecord) {
        Class[] parameterClasses = method.getParameterTypes();
        Object[] arguments = new Object[this.params.length];
        for (int i = 0 ; i < params.length ; ++i) {
            arguments[i] = null == params[i]
                    ? callbackRecord.getCallbackControlPanel()
                    : Proxy.newProxyInstance(
                        method.getDeclaringClass().getClassLoader(),
                        new Class[] {parameterClasses[i]},
                        callbackRecord.newCallbackInvoker(params[i]));
        }
        return arguments;
    }

    public String toString() {
        return this.method.getName();
    }
}
