/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.junit4.support;

import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import org.callbackparams.internal.template.CallbackRecord;

/**
 * Takes callback-records (as a collection of arrays or collections) and
 * test-methods as constructor arguments and iterates through all possible
 * combinations. During iteration the current test-method will be exposed as a
 * CallbackMethod instance and the current record as a CallbackRecord.
 *
 * @author Henrik Kaipe
 */
public class CallbackTestRunIterator {
    
    private final Collection records;
    private final int nbrOfTestMethods;
    private final Iterator methodIterator;
    
    private Iterator currentRecordIterator;
    private CallbackMethod currentTestMethod;
    private CallbackRecord currentCallbackRecord;

    public CallbackTestRunIterator(Collection records, Collection testMethods) {
        this.records = records;
        this.nbrOfTestMethods = testMethods.size();
        this.methodIterator = testMethods.iterator();
    }
    
    public int totalSize() {
        return records.size() * nbrOfTestMethods;
    }

    public boolean next() {
        if (null != currentRecordIterator && currentRecordIterator.hasNext()) {
            final Object nextRecord = currentRecordIterator.next();
            currentCallbackRecord = new CallbackRecord(
                    nextRecord instanceof Object[]
                    ? Arrays.asList((Object[]) nextRecord)
                    : (Collection) nextRecord);
            return true;
            
        } else if (methodIterator.hasNext()) {
            currentTestMethod = new CallbackMethod(
                    (Method) methodIterator.next());
            currentRecordIterator = records.iterator();
            return next();
            
        } else {
            return false;
        }
    }

    public CallbackMethod getCurrentCallbackTestMethod() {
        return currentTestMethod;
    }

    public CallbackRecord getCurrentCallbackRecord() {
        return currentCallbackRecord;
    }
}
