/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.junit4.support;

import java.lang.reflect.Method;

/**
 * @author Henrik Kaipe
 */
public class ExperimentalUtils {
    
    /**
     * Checks whether the specified method is a callback-params method.
     * The method is a callback-params method in case it takes at least one
     * argument and all parameter types are interfaces.
     */
    public static boolean isCallbackParamsMethod(Method m) {
        final Class[] paramTypes = m.getParameterTypes();
        
        if (null == paramTypes || 0 == paramTypes.length) {
            return false;
        }
        
        for (int i = 0 ; i < paramTypes.length ; ++i) {
            if (false == paramTypes[i].isInterface()) {
                return false;
            }
        }
        
        return true;
    }
}
